RUSBoostClassifier — imbalanced-ensemble 0.1.0 documentation

# RUSBoostClassifier

class `imbalanced_ensemble.ensemble.under_sampling.` `RUSBoostClassifier`(**kwargs)

  Random under-sampling integrated in the learning of AdaBoost.

  During learning, the problem of class balancing is alleviated by random under-sampling the sample at each iteration of the boosting algorithm. RUSBoost is originally proposed in [1].

  This implementation extends RUSBoost to support multi-class classification.

  Parameters
    base_estimatorestimator object, default=None

      The base estimator from which the boosted ensemble is built. Support for sample weighting is required, as well as proper `classes_` and `n_classes_` attributes. If `None`, then the base estimator is `DecisionTreeClassifier(max_depth=1)`.

    n_estimatorsint, default=50

      The maximum number of estimators at which boosting is terminated. In case of perfect fit, the learning procedure is stopped early.

    replacementbool, default=False

      Whether or not to sample with replacement.

    learning_ratefloat, default=1.0

      Learning rate shrinks the contribution of each classifier by `learning_rate`. There is a trade-off between `learning_rate` and `n_estimators`.

    algorithm{‘SAMME’, ‘SAMME.R’}, default=’SAMME.R’

      If ‘SAMME.R’ then use the SAMME.R real boosting algorithm. `base_estimator` must support calculation of class probabilities. If ‘SAMME’ then use the SAMME discrete boosting algorithm. The SAMME.R algorithm typically converges faster than SAMME, achieving a lower test error with fewer boosting iterations.

    random_stateint, RandomState instance or None, default=None

      Control the randomization of the algorithm. Within each iteration, a different seed is generated for each sampler. If the base estimator accepts a random_state attribute, a different seed is generated for each instance in the ensemble. Pass an `int` for reproducible output across multiple function calls.

      - If `int`, `random_state` is the seed used by the random number generator;

      - If `RandomState` instance, random_state is the random number generator;

      - If `None`, the random number generator is the `RandomState` instance used by `np.random`.

  Attributes
    base_estimator_estimator

      The base estimator from which the ensemble is grown.

    base_sampler_RandomUnderSampler

      The base sampler.

    estimators_list of classifiers

      The collection of fitted sub-estimators.

    samplers_list of RandomUnderSampler

      The collection of used samplers.

    classes_ndarray of shape (n_classes,)

      The classes labels.

    n_classes_int

      The number of classes.

    estimator_weights_ndarray of shape (n_estimator,)

      Weights for each estimator in the boosted ensemble.

    estimator_errors_ndarray of shape (n_estimator,)

      Classification error for each estimator in the boosted ensemble.

    estimators_n_training_samples_list of ints

      The number of training samples for each fitted base estimators.

    `feature_importances_` ndarray of shape (n_features,)

      The impurity-based feature importances.

  See also

  `SelfPacedEnsembleClassifier`

    Ensemble with self-paced dynamic under-sampling.

  `OverBoostClassifier`

    Random over-sampling integrated AdaBoost.

  `UnderBaggingClassifier`

    Bagging with intergrated random under-sampling.

  Notes

  See Basic usage example of imbalanced_ensemble for an example.

  References

  1

    Seiffert, C., Khoshgoftaar, T. M., Van Hulse, J., & Napolitano, A. “RUSBoost: A hybrid approach to alleviating class imbalance.” IEEE Transactions on Systems, Man, and Cybernetics-Part A: Systems and Humans 40.1 (2010): 185-197.

  Examples

      >>> from imbalanced_ensemble.ensemble import RUSBoostClassifier
      >>> from sklearn.datasets import make_classification
      >>>
      >>> X, y = make_classification(n_samples=1000, n_classes=3,
      ...                            n_informative=4, weights=[0.2, 0.3, 0.5],
      ...                            random_state=0)
      >>> clf = RUSBoostClassifier(random_state=0)
      >>> clf.fit(X, y)
      RUSBoostClassifier(...)
      >>> clf.predict(X)
      array([...])

  Methods

  `decision_function`(X) | Compute the decision function of `X`.
  `fit`(X, y, *[, sample_weight, target_label, …]) | Build a RUSBoost classifier from the training set (X, y).
  `get_params`([deep]) | Get parameters for this estimator.
  `predict`(X) | Predict classes for X.
  `predict_log_proba`(X) | Predict class log-probabilities for X.
  `predict_proba`(X) | Predict class probabilities for X.
  `score`(X, y[, sample_weight]) | Return the mean accuracy on the given test data and labels.
  `set_params`(**params) | Set the parameters of this estimator.
  `staged_decision_function`(X) | Compute decision function of `X` for each boosting iteration.
  `staged_predict`(X) | Return staged predictions for X.
  `staged_predict_proba`(X) | Predict class probabilities for X.
  `staged_score`(X, y[, sample_weight]) | Return staged scores for X, y.

  `decision_function`(X)

    Compute the decision function of `X`.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Returns
      scorendarray of shape of (n_samples, k)

        The decision function of the input samples. The order of outputs is the same of that of the classes_ attribute. Binary classification is a special cases with `k == 1`, otherwise `k==n_classes`. For binary classification, values closer to -1 or 1 mean more like the first or second class in `classes_`, respectively.

  property `feature_importances_`

    The impurity-based feature importances.

    The higher, the more important the feature. The importance of a feature is computed as the (normalized) total reduction of the criterion brought by that feature. It is also known as the Gini importance.

    Warning: impurity-based feature importances can be misleading for high cardinality features (many unique values). See `sklearn.inspection.permutation_importance()` as an alternative.

    Returns
      feature_importances_ndarray of shape (n_features,)

        The feature importances.

  `fit`(X, y, *, sample_weight=None, target_label: int = None, n_target_samples: int = None, balancing_schedule: str = 'uniform', eval_datasets: dict = None, eval_metrics: dict = None, train_verbose: bool = True)

    Build a RUSBoost classifier from the training set (X, y).

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. DOK and LIL are converted to CSR.

      yarray-like of shape (n_samples,)

        The target values (class labels).

      sample_weightarray-like of shape (n_samples,), default=None

        Sample weights. If None, the sample weights are initialized to `1 / n_samples`.

      target_labelint, default=None

        Specify the class targeted by the under-sampling. All other classes that have more samples than the target class will be considered as majority classes. They will be under-sampled until the number of samples is equalized. The remaining minority classes (if any) will stay unchanged.

      n_target_samplesint or dict, default=None

        Specify the desired number of samples (of each class) after the under-sampling.

        - If `int`, all classes that have more than the `n_target_samples` samples will be under-sampled until the number of samples is equalized.

        - If `dict`, the keys correspond to the targeted classes. The values correspond to the desired number of samples for each targeted class.

      balancing_schedulestr, or callable, default=’uniform’

        Scheduler that controls how to sample the data set during the ensemble training process.

        - If `str`, using the predefined balancing schedule. Possible choices are:

          > - `'uniform'`: resample to target distribution for all base estimators;

          > - `'progressive'`: The resample class distributions are progressive interpolation between the original and the target class distribution. Example: For a class \(c\), say the number of samples is \(N_{c}\) and the target number of samples is \(N'_{c}\). Suppose that we are training the \(t\)-th base estimator of a \(T\)-estimator ensemble, then we expect to get \((1-\frac{t}{T}) \cdot N_{c} + \frac{t}{T} \cdot N'_{c}\) samples after resampling;

        - If callable, function takes 4 positional arguments with order (`'origin_distr'`: `dict`, `'target_distr'`: `dict`, `'i_estimator'`: `int`, `'total_estimator'`: `int`), and returns a `'result_distr'`: `dict`. For all parameters of type `dict`, the keys of type `int` correspond to the targeted classes, and the values of type `str` correspond to the (desired) number of samples for each class.

      eval_datasetsdict, default=None

        Dataset(s) used for evaluation during the ensemble training process. The keys should be strings corresponding to evaluation datasets’ names. The values should be tuples corresponding to the input samples and target values.

        Example: `eval_datasets = {'valid' : (X_valid, y_valid)}`

      eval_metricsdict, default=None

        Metric(s) used for evaluation during the ensemble training process.

        - If `None`, use 3 default metrics:

          > - `'acc'`: `sklearn.metrics.accuracy_score()`

          > - `'balanced_acc'`: `sklearn.metrics.balanced_accuracy_score()`

          > - `'weighted_f1'`: `sklearn.metrics.f1_score(average='weighted')`

        - If `dict`, the keys should be strings corresponding to evaluation metrics’ names. The values should be tuples corresponding to the metric function (`callable`) and additional kwargs (`dict`).

          > - The metric function should at least take 2 positional arguments `y_true`, `y_pred`, and returns a `float` as its score.

          > - The metric additional kwargs should specify the additional arguments that need to be passed into the metric function.

        Example: `{'weighted_f1': (sklearn.metrics.f1_score, {'average': 'weighted'})}`

      train_verbosebool, int or dict, default=True

        Controls the verbosity during ensemble training/fitting.

        - If `bool`: `False` means disable training verbose. `True` means print training information to sys.stdout use default setting:

          > - `'granularity'` : `int(n_estimators/10)`

          > - `'print_distribution'` : `True`

          > - `'print_metrics'` : `True`

        - If `int`, print information per `train_verbose` rounds.

        - If `dict`, control the detailed training verbose settings. They are:

          > - `'granularity'`: corresponding value should be `int`, the training information will be printed per `granularity` rounds.

          > - `'print_distribution'`: corresponding value should be `bool`, whether to print the data class distribution after resampling. Will be ignored if the ensemble training does not perform resampling.

          > - `'print_metrics'`: corresponding value should be `bool`, whether to print the latest performance score. The performance will be evaluated on the training data and all given evaluation datasets with the specified metrics.

        Warning

        Setting a small `'granularity'` value with `'print_metrics'` enabled can be costly when the training/evaluation data is large or the metric scores are hard to compute. Normally, one can set `'granularity'` to `n_estimators/10`.

    Returns
      selfobject

        Returns self.

  `get_params`(deep=True)

    Get parameters for this estimator.

    Parameters
      deepbool, default=True

        If True, will return the parameters for this estimator and contained subobjects that are estimators.

    Returns
      paramsdict

        Parameter names mapped to their values.

  `predict`(X)

    Predict classes for X.

    The predicted class of an input sample is computed as the weighted mean prediction of the classifiers in the ensemble.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Returns
      yndarray of shape (n_samples,)

        The predicted classes.

  `predict_log_proba`(X)

    Predict class log-probabilities for X.

    The predicted class log-probabilities of an input sample is computed as the weighted mean predicted class log-probabilities of the classifiers in the ensemble.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Returns
      pndarray of shape (n_samples, n_classes)

        The class probabilities of the input samples. The order of outputs is the same of that of the classes_ attribute.

  `predict_proba`(X)

    Predict class probabilities for X.

    The predicted class probabilities of an input sample is computed as the weighted mean predicted class probabilities of the classifiers in the ensemble.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Returns
      pndarray of shape (n_samples, n_classes)

        The class probabilities of the input samples. The order of outputs is the same of that of the classes_ attribute.

  `score`(X, y, sample_weight=None)

    Return the mean accuracy on the given test data and labels.

    In multi-label classification, this is the subset accuracy which is a harsh metric since you require for each sample that each label set be correctly predicted.

    Parameters
      Xarray-like of shape (n_samples, n_features)

        Test samples.

      yarray-like of shape (n_samples,) or (n_samples, n_outputs)

        True labels for X.

      sample_weightarray-like of shape (n_samples,), default=None

        Sample weights.

    Returns
      scorefloat

        Mean accuracy of `self.predict(X)` wrt. y.

  `set_params`(**params)

    Set the parameters of this estimator.

    The method works on simple estimators as well as on nested objects (such as `Pipeline`). The latter have parameters of the form `<component>__<parameter>` so that it’s possible to update each component of a nested object.

    Parameters
      **paramsdict

        Estimator parameters.

    Returns
      selfestimator instance

        Estimator instance.

  `staged_decision_function`(X)

    Compute decision function of `X` for each boosting iteration.

    This method allows monitoring (i.e. determine error on testing set) after each boosting iteration.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Yields
      scoregenerator of ndarray of shape (n_samples, k)

        The decision function of the input samples. The order of outputs is the same of that of the classes_ attribute. Binary classification is a special cases with `k == 1`, otherwise `k==n_classes`. For binary classification, values closer to -1 or 1 mean more like the first or second class in `classes_`, respectively.

  `staged_predict`(X)

    Return staged predictions for X.

    The predicted class of an input sample is computed as the weighted mean prediction of the classifiers in the ensemble.

    This generator method yields the ensemble prediction after each iteration of boosting and therefore allows monitoring, such as to determine the prediction on a test set after each boost.

    Parameters
      Xarray-like of shape (n_samples, n_features)

        The input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Yields
      ygenerator of ndarray of shape (n_samples,)

        The predicted classes.

  `staged_predict_proba`(X)

    Predict class probabilities for X.

    The predicted class probabilities of an input sample is computed as the weighted mean predicted class probabilities of the classifiers in the ensemble.

    This generator method yields the ensemble predicted class probabilities after each iteration of boosting and therefore allows monitoring, such as to determine the predicted class probabilities on a test set after each boost.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

    Yields
      pgenerator of ndarray of shape (n_samples,)

        The class probabilities of the input samples. The order of outputs is the same of that of the classes_ attribute.

  `staged_score`(X, y, sample_weight=None)

    Return staged scores for X, y.

    This generator method yields the ensemble score after each iteration of boosting and therefore allows monitoring, such as to determine the score on a test set after each boost.

    Parameters
      X{array-like, sparse matrix} of shape (n_samples, n_features)

        The training input samples. Sparse matrix can be CSC, CSR, COO, DOK, or LIL. COO, DOK, and LIL are converted to CSR.

      yarray-like of shape (n_samples,)

        Labels for X.

      sample_weightarray-like of shape (n_samples,), default=None

        Sample weights.

    Yields
      zfloat

## Examples using `imbalanced_ensemble.ensemble.under_sampling.RUSBoostClassifier`

Basic usage example of imbalanced_ensemble